import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
} from 'd3';

const url = `https://gist.githubusercontent.com/nitanagdeote/bb42bb3798faf2a55be1e4a6e82ca493/raw/568207cf6758201f5c53ff73c3d8ed85e342ce2d/.csv`;

const svg = select('.my_bar');

const width = +svg.attr('width');
const height = +svg.attr('height');
const xAxisLabelText = 'Range(miles)';
const titleText =
  'Comparative Analysis: EV Range with and without GTC Range Extender';
const margin = {
  top: 120,
  right: 80,
  bottom: 77,
  left: 200,
};

//////////////////////////////////////////////////

////////////////////////////////////////////////////

const render = (data) => {
  const xValue = (d) => d['Current Range(miles)'];
  const yValue = (d) => d['Vehicle'];
  const zValue = (d) => d['New Range(miles)'];

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleLinear()
    .domain([0, max(data, zValue)])
    .range([0, innerWidth]);

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(0.7);

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const xAxisTickFormat = (number) => format('')(number);

  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat)
    .tickSize(-innerHeight);

  g.append('g')
    .call(axisLeft(yScale))
    .selectAll('.domain, .tick line')
    .remove();

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 65)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabelText);

  g.selectAll('.bar')
    .data(data)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(yValue(d)))
    .attr('width', (d) => xScale(xValue(d)))
    .attr('height', yScale.bandwidth())
    .attr('fill', 'green')
    .append('title')
    .text(
      (d) =>
        `${d['Vehicle']}  :  ${d['Current Range(miles)']}`,
    );

  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .attr('x', -120)
    .text(titleText);

  g.selectAll('.bar')
    .data(data)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(yValue(d)) - 15)
    .attr('width', (d) => xScale(zValue(d)))
    .attr('height', yScale.bandwidth())
    .attr('fill', 'orange')

    .append('title')
    .text(
      (d) =>
        `${d['Vehicle']}  :  ${d['New Range(miles)']} with GTC`,
    );

  //Labels

  const labelG = svg.append('g')
    .attr('transform', `translate(0,${innerHeight/3})`);
    ;
  const labelRectG = labelG.append('g');
  const gLabelText = labelG.append('g');

  const xOffset = 100;
  const yOffset = 10;

  labelRectG
    .append('rect')
    .attr('x', innerWidth + xOffset)
    .attr('y', innerHeight / 3 - yOffset)
    .attr('stroke', 'green')
    .attr('stroke-width', '5px')
    .attr('fill', 'none')
    .attr('width', '20px')
    .attr('height', '20px');
  labelRectG
    .append('rect')
    .attr('x', innerWidth + xOffset)
    .attr('y', innerHeight / 3 - 30 - yOffset)
    .attr('stroke', 'orange')
    .attr('stroke-width', '5px')
    .attr('fill', 'none')
    .attr('width', '20px')
    .attr('height', '20px');

  gLabelText
    .append('text')
    .attr('x', innerWidth + 140)
    .attr('y', innerHeight / 3 - 15 - yOffset)
    .text('GTC Range');
  gLabelText
    .append('text')
    .attr('x', innerWidth + 140)
    .attr('y', innerHeight / 3 + 15 - yOffset)

    .text('Current Range');

  /////////////////////////////////////
  const textG1 = g.append('g');
  const textG2 = g.append('g');
  textG1;
  textG1
    .selectAll('text')
    .data(data)
    .enter()
    .append('text')

    .attr('y', (d) => yScale(yValue(d)) - 5)
    .attr('x', (d) => xScale(zValue(d)) + 10)

    .text((d) => d['New Range(miles)']);

  textG2
    .selectAll('text')
    .data(data)
    .enter()
    .append('text')

    .attr('y', (d) => yScale(yValue(d)) + 13)
    .attr('x', (d) => xScale(xValue(d)) + 5)

    .text((d) => d['Current Range(miles)']);
};

////////////////////////////////////////////////////
d3.csv(url).then((data) => {
  data.forEach((d) => {
    d['Current Range(miles)'] = +d['Current Range(miles)'];
    d['New Range(miles)'] = +d['New Range(miles)'];
  });
  render(data);
});
